package utils

import (
	"fmt"
	"strings"
)

// GetActivityName 根据步骤类型获取对应的 Activity 函数名
// 参数:
//   - stepType: 数据库中的步骤类型，如 "api", "ui" 等
//
// 返回值:
//   - activityName: 对应的 Temporal Activity 函数名
//   - error: 如果步骤类型不支持则返回错误
func GetActivityName(stepType string) (string, error) {
	// 标准化输入：转为大写并去除空格
	normalizedStepType := strings.ToUpper(strings.TrimSpace(stepType))

	// 步骤类型到 Activity 名称的映射表
	stepTypeToActivity := map[string]string{
		// ================== 测试类型 Activity ==================
		"API":    "RunApiTest",    // API 接口测试
		"UI":     "RunUiTest",     // UI 自动化测试
		"SQL":    "RunSqlTest",    // SQL 执行sql语句
		"PYTHON": "RunPythonTest", // PYTHON 执行python脚本处理数据
	}

	// 查找对应的 Activity 名称
	activityName, exists := stepTypeToActivity[normalizedStepType]
	if !exists {
		// 返回详细的错误信息，包含支持的类型列表
		supportedTypes := make([]string, 0, len(stepTypeToActivity))
		for stepType := range stepTypeToActivity {
			supportedTypes = append(supportedTypes, stepType)
		}

		return "", fmt.Errorf("unsupported step type '%s'. Supported types: %v",
			stepType, supportedTypes)
	}

	return activityName, nil
}
